/*
firetalk-int.h - FireTalk wrapper declarations
Copyright (C) 2000 Ian Gulliver

This program is free software; you can redistribute it and/or modify
it under the terms of version 2 of the GNU General Public License as
published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _FIRETALK_INT_H
#define _FIRETALK_INT_H

typedef void (*ptrtofnct)(void *, const int, const int,...);

typedef struct s_firetalk_buddy {
	struct s_firetalk_buddy *next;
	char *nickname;
	int online;
	int away;
} s_firetalk_buddy;

typedef struct s_firetalk_deny {
	struct s_firetalk_deny *next;
	char *nickname;
} s_firetalk_deny;

typedef struct s_firetalk_handle {
	void *handles[FP_MAX][FIRETALK_MAX_CONNECTIONS];
	ptrtofnct callbacks[FC_MAX];
	struct s_firetalk_handle *next;
	struct s_firetalk_handle *prev;
	struct s_firetalk_buddy *buddy_head[FP_MAX][FIRETALK_MAX_CONNECTIONS];
	struct s_firetalk_deny *deny_head[FP_MAX][FIRETALK_MAX_CONNECTIONS];
} s_firetalk_handle;

void firetalk_callback_im_getmessage(const int protocol, void * const c, const char * const sender, const int automessage, const char * const message);
void firetalk_callback_im_buddyonline(const int protocol, void * const c, const char * const nickname);
void firetalk_callback_im_buddyoffline(const int protocol, void * const c, const char * const nickname);
void firetalk_callback_im_buddyaway(const int protocol, void * const c,  const char * const nickname);
void firetalk_callback_im_buddyunaway(const int protocol, void * const c, const char * const nickname);
void firetalk_callback_error(const int protocol, void * const c, const int error, const char * const roomoruser);
void firetalk_callback_disconnect(const int protocol, void * const c, const int error);
void firetalk_callback_gotinfo(const int protocol, void * const c, const char * const nickname, const char * const info, const int warning, const int idle);
void firetalk_callback_idleinfo(const int protocol, void * const c, char const * const nickname, const long idletime);
void firetalk_callback_doinit(const int protocol, void * const c);
void firetalk_callback_setidle(const int protocol, void * const c, long * const idle);
void firetalk_callback_eviled(const int protocol, void * const c, const int newevil, const char * const eviler);
void firetalk_callback_chat_joined(const int protocol, void * const c, const char * const room);
void firetalk_callback_chat_left(const int protocol, void * const c, const char * const room);
void firetalk_callback_chat_getmessage(const int protocol, void * const c, const char * const room, const char * const from, const char * const message);
void firetalk_callback_chat_getwhisper(const int protocol, void * const c, const char * const room, const char * const from, const char * const message);
void firetalk_callback_chat_invited(const int protocol, void * const c, const char * const room, const char * const from, const char * const message);
void firetalk_callback_chat_user_joined(const int protocol, void * const c, const char * const room, const char * const who);
void firetalk_callback_chat_user_left(const int protocol, void * const c, const char * const room, const char * const who);

struct s_firetalk_handle *firetalk_find_handle(const int protocol, void * const c);
int firetalk_find_index(const struct s_firetalk_handle * const conn, const int protocol, void * const c);

#endif
