/*
firetalk.h - FireTalk wrapper declarations
Copyright (C) 2000 Ian Gulliver

This program is free software; you can redistribute it and/or modify
it under the terms of version 2 of the GNU General Public License as
published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _FIRETALK_H
#define _FIRETALK_H

#include <unistd.h>
#include <sys/time.h>

#define LIBFIRETALK_VERSION "0.0.8"

#define FP_AIMTOC 0
#define FP_MAX 1

#define FIRETALK_MAX_CONNECTIONS 16

/* Firetalk functions */
void *firetalk_create_handle();
void firetalk_destroy_handle(void * const c);
void firetalk_perror(const char * const string);
int firetalk_disconnect(void * const c, const int protocol, const int connection);
int firetalk_signon(void * const c, const int protocol, const int connection, const char * const server, const int port, const char * const username, const char * const password);
int firetalk_register_callback(void * const c, const int type, void (*function)(void *, const int, const int, ...));
#define FC_IM_GETMESSAGE 1
		/* void *connection, int protocol, int connection, char *sender, int automessage_flag, char *message */
#define FC_ERROR 2
		/* void *connection, int protocol, int connection, int error, char *roomoruser (room or user that error applies to, null if none) */
#define FC_DISCONNECT 3
		/* void *connection, int protocol, int connection */
#define FC_GOTINFO 4
		/* void *connection, int protocol, int connection, char *nickname, char *info, int warning, int idle */
#define FC_IM_BUDDYONLINE 5
		/* void *connection, int protocol, int connection, char *nickname */
#define FC_IM_BUDDYOFFLINE 6
		/* void *connection, int protocol, int connection, char *nickname */
#define FC_IM_BUDDYAWAY 7
		/* void *connection, int protocol, int connection, char *nickname */
#define FC_IM_BUDDYUNAWAY 8
		/* void *connection, int protocol, int connection, char *nickname */
#define FC_IM_LISTBUDDY 9
		/* void *connection, int protocol, int connection, char *nickname, char online, char away */
#define FC_IDLEINFO 10
		/* void *connection, int protocol, int connection, char *nickname, long idletime */
#define FC_DOINIT 11
		/* void *connection, int protocol, int connection */
#define FC_SETIDLE 12
		/* void *connection, int protocol, int connection, long *idle */
#define FC_EVILED 13
		/* void *connection, int protocol, int connection, int newevil, char *eviler */
#define FC_CHAT_JOINED 14
		/* void *connection, int protocol, int connection, char *room */
#define FC_CHAT_LEFT 15
		/* void *connection, int protocol, int connection, char *room */
#define FC_CHAT_GETMESSAGE 16
		/* void *connection, int protocol, int connection, char *room, char *from, char *message */
#define FC_CHAT_GETWHISPER 17
		/* void *connection, int protocol, int connection, char *room, char *from, char *message */
#define FC_CHAT_INVITED 18
		/* void *connection, int protocol, int connection, char *room, char *from, char *message */
#define FC_CHAT_USER_JOINED 19
		/* void *connection, int protocol, int connection, char *room, char *who */
#define FC_CHAT_USER_LEFT 20
		/* void *connection, int protocol, int connection, char *room, char *who */
#define FC_MAX 21

int firetalk_im_add_buddy(void * const c, const int protocol, const int connection, const char * const nickname);
int firetalk_im_remove_buddy(void * const c, const int protocol, const int connection, const char * const nickname);
int firetalk_im_add_deny(void * const c, const int protocol, const int connection, const char * const nickname);
int firetalk_im_remove_deny(void * const c, const int protocol, const int connection, const char * const nickname);
int firetalk_im_upload_buddies(void * const c, const int protocol, const int connection);
int firetalk_im_upload_denies(void * const c, const int protocol, const int connection);
int firetalk_im_send_message(void * const c, const int protocol, const int connection, const char * const dest, const char * const message, const int auto_flag);
int firetalk_im_list_buddies(void * const c, const int protocol, const int connection);
int firetalk_im_evil(void * const c, const int protocol, const int connection, const char * const who);

int firetalk_chat_join(void * const c, const int protocol, const int connection, const char * const room);
int firetalk_chat_part(void * const c, const int protocol, const int connection, const char * const room);
int firetalk_chat_send_message(void * const c, const int protocol, const int connection, const char * const room, const char * const message);
int firetalk_chat_invite(void * const c, const int protocol, const int connection, const char * const room, const char * const who, const char * const message);
int firetalk_chat_whisper(void * const c, const int protocol, const int connection, const char * const room, const char * const who, const char * const message);

int firetalk_im_internal_add_buddy(void * const c, const int protocol, const int connection, const char * const nickname);
int firetalk_im_internal_add_deny(void * const c, const int protocol, const int connection, const char * const nickname);
int firetalk_im_internal_remove_buddy(void * const c, const int protocol, const int connection, const char * const nickname);
int firetalk_im_internal_remove_deny(void * const c, const int protocol, const int connection, const char * const nickname);

int firetalk_save_config(void * const c, const int protocol, const int connection);
int firetalk_get_info(void * const c, const int protocol, const int connection, const char * const nickname);
int firetalk_set_info(void * const c, const int protocol, const int connection, const char * const info);
int firetalk_set_away(void * const c, const int protocol, const int connection, const char * const message);
int firetalk_select(void * const c);
int firetalk_select_custom(void * const c, int n, fd_set *fd_read, fd_set *fd_write, fd_set *fd_except, struct timeval *timeout);

#ifndef FIRETALK
extern int firetalkerror;
extern char *firetalkerrors[];
extern char *firetalkprotocols[];
#endif

#define FE_SUCCESS                      0
#define FE_CONNECT                      1
#define FE_NOMATCH                      2
#define FE_PACKET                       3
#define FE_BADUSERPASS                  4
#define FE_SEQUENCE                     5
#define FE_FRAMETYPE                    6
#define FE_PACKETSIZE                   7
#define FE_SERVER                       8
#define FE_UNKNOWN                      9
#define FE_BLOCKED                     10
#define FE_WIERDPACKET                 11
#define FE_CALLBACKNUM                 12
#define FE_BADUSER                     13
#define FE_NOTFOUND                    14
#define FE_DISCONNECT                  15
#define FE_SOCKET                      16
#define FE_RESOLV                      17
#define FE_VERSION                     18
#define FE_USERUNAVAILABLE             19
#define FE_USERINFOUNAVAILABLE         20
#define FE_TOOFAST                     21
#define FE_ROOMUNAVAILABLE             22
#define FE_INCOMINGERROR               23
#define FE_USERDISCONNECT              24
#define FE_INVALIDFORMAT               25
#define FE_IDLEFAST                    26
#define FE_BADROOM                     27
#define FE_BADMESSAGE                  28
#define FE_BADPROTO                    29

#endif
