/*  _ __   __ _ ___ __  __
** | '_ \ / _` |_ _|  \/  | nAIM PRERELEASE
** | | | | (_| || || |\/| | Copyright 1998 Daniel Reed <n@ml.org>
** |_| |_|\__,_|___|_|  |_| ncurses-based, FAIM-based AIM client
**
** Based on ProtoFAIM v0.xx.xxplxx (faimtest.c) by Adam Fritzler
** FAIM Copyright 1998 Adam Fritzler <afritz@iname.com>
** FAIM uses code Copyright 1998 Brock Wilcox <awwaiid@iname.com>
*/
#ifndef naim_h
#define naim_h	1

#include <config.h>
#include <../secs/secs.h>
#include <nfaim.h>

#include <stdarg.h>

#ifdef HAVE_CTYPE_H
# include <ctype.h>
#endif
#ifdef HAVE_ERRNO_H
# include <errno.h>
#endif
#ifdef HAVE_SYS_ERRNO_H
# include <sys/errno.h>
#endif
#ifdef HAVE_STDLIB_H
# include <stdlib.h>
#endif

#define NCURSES_CONST const
#ifdef HAVE_NCURSES_H
# include <ncurses.h>
#else
# ifdef HAVE_CURSES_H
#  include <curses.h>
# else
#  error Unable to locate ncurses.h; please see README.ncurses
# endif
#endif

#ifndef NO_HISTORY
# ifdef HAVE_READLINE_HISTORY_H
#  include <readline/history.h>
# else
#  ifdef HAVE_HISTORY_H
#   include <history.h>
#  else
#   error Unable to locate history.h; please see README.history
#  endif
# endif
#endif

#ifdef HAVE_ASSERT_H
# include <assert.h>
#else
# define assert(x)
#endif

#ifdef HAVE_TIME_H
# include <time.h>
# ifdef HAVE_SYS_TIME_H
#  ifdef TIME_WITH_SYS_TIME
#   include <sys/time.h>
#  endif
# endif
#else
# ifdef HAVE_SYS_TIME_H
#  include <sys/time.h>
# endif
#endif

// no, this isn't where you configure the colours; this just sets up the
// pair numerics, the pairs are associated with actual colours in win.c
#define HELP_HELP	10
#define HELP_TEXT	11

#define IMWIN_BGCOLOR	20
#define IMWIN_TEXT_U	21
#define IMWIN_TEXT_R	22
#define IMWIN_STATUS	23

#define INPUT_BGCOLOR	30
#define INPUT_TEXT	31
#define INPUT_TEXT_ALT	32

#define BUDDY_BGCOLOR	40
#define BUDDY_TEXT	41
#define BUDDY_WAITING	42
#define BUDDY_OFFLINE	43
#define BUDDY_NONBUDDY	44
#define BUDDY_INFO	45
#define BUDDY_AWAY	46
#define BUDDY_IDLE	47
#define BUDDY_CHATROOM	48

#define STAT_BGCOLOR	50
#define STAT_TEXT	51

#define BACKDROP	60

#define CONIO_MAXPARMS	10

typedef struct naimses_ts {
       char    sn[MAXSNLEN+1];
       int     lag, connected;
       time_t  online;
} naimses_t;

typedef struct buddywin_ts {
	char	winname[/*MAXSNLEN+1*/31];
	char	waiting,	// text waiting to be read (overrides
				// offline and isbuddy in bupdate())
		offline,	// window currently points to offline person
		notbuddy,	// window is for a non-buddy
		keepafterso,	// keep window open after buddy signs off?
		isaway,		// is the buddy away?
		isidle,		// is the buddy idle for more than some threshhold?
		ischat,		// chat window
		*cryptkey;	// crypto key phrase
	WINDOW	*win;
	struct buddywin_ts	*next;
	time_t	informed;
} buddywin_t;

typedef struct buddylist_ts {
	char	screenname[MAXSNLEN+1],
		group[21],
		notes[41];
	struct buddylist_ts	*next;
} buddylist_t;

typedef struct ignorelist_ts {
	char	screenname[MAXSNLEN+1],
		notes[128];
	struct ignorelist_ts	*next;
	time_t	informed;
} ignorelist_t;

typedef struct {
	char	screenname[MAXSNLEN+1],
		password[16],
		logfile[128],
		*profile,
		nolog,
		nobackcol,
		nolagcheck,
		sortby,
		rcreadonly;
	buddylist_t	*buddies;
	ignorelist_t	*idiots;
	int	cols;
	struct {
		char	fore, back;
	} colors[64];
	struct {
		int	startx, starty,
			widthx, widthy;
	} wstatus, winput, wbuddy;
} faimconf_t;

/* aimio.c */
//void	naim_prof(char *);
//void	naim_budprof(struct aim_session_t *sess, struct aim_conn_t *);

/* buddy.c */
void	bsetup(void);
void	bnewwin(char *);
void	bupdate(void);
void	bcoming(char *, int, int);
void	bgoing(char *);
void	bremove(buddywin_t *);
buddywin_t	*bgetwin(const char *);
void	bclearall(void);

/* conio.c */
int	conio_handlecmd(char *);
void	gotkey(int);

/* events.c */
void	updateidletime(void);
void	event_handle(time_t);

/* hamster.c */
void	logim(const char *, const char *, const char *);
char	*naim_xor(char *, const char *, const char *);
void	naim_send_im(/*const*/ char *, /*const*/ char *, const char *);
void	naim_send_im_away(/*const*/ char *, /*const*/ char *);
void	setaway(void);
void	unsetaway(void);
void	sendaway(char *);

/* helpcmd.c */
void	help_printhelp(char *);

/* hwprintf.c */
int	hwprintf(WINDOW *, int, const char *, ...);

/* rc.c */
void	killconf(faimconf_t *);
buddylist_t	*rgetlist(faimconf_t *, const char *);
buddylist_t	*raddbuddy(faimconf_t *, const char *, const char *,
	const char *);
void	raddidiot(faimconf_t *, const char *, const char *);
char	*rmakebuddyar(faimconf_t *);
int	naim_read_config(const char *);

/* rodents.c */
int	aimcmp(const char *, const char *);

/* script.c */
int	script_doalias(char *, char *);

/* set.c */
void	set_setvar(char *);

/* win.c */
void	wsetp(void);
void	wsetup(void);
int	wcols(faimconf_t *);
void	winitwin(WINDOW *, int);
int	wprintf(WINDOW *, int, int, const char *, ...);
void	wshutitdown(void);

#endif /* naim_h */
