/*  ___  ___  ___ ___
** / __|/ _ \/ __/ __| secs
** \__ \  __/ (__\__ \ Copyright 1999 Daniel Reed <n@ml.org>
** |___/\___|\___|___/ Simple Embedded Client Scripting
*/
#include <secs.h>

extern int	(*secs_client_cmdhandler)(char *);

struct {
	secs_block_t	*secs_root;
} sd;

secs_block_t	*secs_block_getroot(void) {
	return(sd.secs_root);
}

int	secs_init(void) {
	secs_mem_init();
	secs_var_init();
	secs_block_init();
	secs_script_init();
	sd.secs_root = secs_block_create(NULL, "root");
	assert(sd.secs_root != NULL);
	return(1);
}

void	secs_handle(char *line) {
	debug_printf("secs_handle(``%s'')\n", line);
	(*secs_client_cmdhandler)(line);
}

int	secs_makevar(char *name, char *value, char type) {
	secs_var_t	*var = NULL;

	debug_printf("secs_makevar(``%s'', ``%s'', `%c')\n", name, value, type);
	if ((name == NULL) || (value == NULL) || (type == 0))
		return(secs_var_set(var, value));

	if ((var = secs_var_find(NULL, name)) != NULL)
		return(0);

	if (toupper(type) == 'I')
		var = secs_var_create(name, secs_var_set_int);
	else if (toupper(type) == 'S')
		var = secs_var_create(name, secs_var_set_str);
	else if (toupper(type) == 'B')
		var = secs_var_create(name, secs_var_set_switch);
	else
		return(0);
	secs_var_set(var, value);
	return(secs_block_var_add(sd.secs_root, var));
}

int	secs_setvar(char *name, char *val) {
	secs_var_t	*var = NULL;

	debug_printf("secs_setvar(``%s'', ``%s'')\n", name, val);
	if ((name == NULL) || (val == NULL))
		return(0);
	if ((var = secs_var_find(NULL, name)) == NULL)
		return(secs_makevar(name, val, 'S'));
	return(secs_var_set(var, val));
}

char	*secs_getvar(char *name) {
	secs_var_t	*var = NULL;

	debug_printf("secs_getvar(``%s'')\n", name);
	if ((var = secs_var_find(NULL, name)) == NULL)
		return(NULL);
	return(var->val);
}

int	secs_getvar_int(char *name) {
	char	*p = secs_getvar(name);

	if (p != NULL)
		return(atoi(p));
	return(0);
}

char	*secs_listvars(int i, size_t *length) {
	static secs_var_t	*var;
	char	*tmp = NULL;

	if (i == 0) {
		var = sd.secs_root->variables;
		return(NULL);
	}
	if (var == NULL)
		return(NULL);
	tmp = var->name;
	if (length != NULL)
		*length = var->length;
	var = var->next;
	return(tmp);
}
