/*  _ __   __ _ ___ __  __
** | '_ \ / _` |_ _|  \/  | naim
** | | | | (_| || || |\/| | Copyright 1998, 1999 Daniel Reed <n@ml.org>
** |_| |_|\__,_|___|_|  |_| ncurses-based, FAIM-based AIM client
** 
** FAIM Copyright 1998 Adam Fritzler <afritz@iname.com>
** FAIM uses code Copyright 1998 Brock Wilcox <awwaiid@iname.com>
*/
#include <naim.h>

extern WINDOW	*win_buddy, *win_status;
extern faimconf_t	faimconf;
extern naimses_t       naimses;
extern int	iamaway;
extern time_t	idletime;

buddywin_t	nullbwin,
		*curbwin;

void	bsetup(void) {
	curbwin = &nullbwin;
	bzero(curbwin, sizeof(buddywin_t));
	strncpy(curbwin->winname, "*] Status Win [*", MAXSNLEN);
	curbwin->win = win_status;
	curbwin->next = curbwin;
	bupdate();
}

void	bupdate(void) {
	buddywin_t	*bwin = curbwin;
	time_t  now, t;
	struct tm       *tmptr;

	assert(bwin != NULL);
	bwin->waiting = 0;

	now = time(NULL);
	assert(now != 0);
	tmptr = localtime(&now);
	assert(tmptr != NULL);
	werase(win_buddy);

	do {
		char	*buf = bwin->winname;

		if (bwin->waiting)
			wprintf(win_buddy, BUDDY_WAITING, 1,
				"(%16.16s)", buf);
		else if (bwin->ischat)
			wprintf(win_buddy, BUDDY_CHATROOM, 0,
				"|%16.16s|", buf);
		else if (bwin->offline)
			wprintf(win_buddy, BUDDY_OFFLINE, 0,
				"<%16.16s>", buf);
		else if (bwin->notbuddy)
			wprintf(win_buddy, BUDDY_NONBUDDY, 1,
				"{%16.16s}", buf);
		else if (bwin->isaway)
			wprintf(win_buddy, BUDDY_AWAY, 1,
				"[%16.16s]", buf);
		else if (bwin->isidle)
			wprintf(win_buddy, BUDDY_IDLE, 1,
				"[%16.16s]", buf);
		else
			wprintf(win_buddy, BUDDY_TEXT, 1,
				"[%16.16s]", buf);
	} while ((bwin = bwin->next) != curbwin);
	wprintf(win_buddy, BUDDY_INFO, 1, "\n");
	wprintf(win_buddy, BUDDY_INFO, 1, "naim      %02i:%02i %s",
		(((tmptr->tm_hour)+23)%12)+1, tmptr->tm_min,
		((tmptr->tm_hour)>11)?"PM":"AM");
	if (faimconf.nolagcheck == 0)
		wprintf(win_buddy, BUDDY_INFO, 1, "Lag %12i s",
			naimses.lag);
	t = (now - naimses.online)/60;
	wprintf(win_buddy, BUDDY_INFO, 1, "Online %9i", t/60);
	wprintf(win_buddy, BUDDY_INFO, 0, "%02i", t%60);
//	wprintf(win_buddy, BUDDY_INFO, 1, "Online %9i %s",
//		((t/60>1)?(t/60):t), ((t/60>1)?"h":"m"));
	t = (now - idletime)/60;
	wprintf(win_buddy, BUDDY_INFO, 1, "Idle %11i %s",
		((t/60>1)?(t/60):t), ((t/60>1)?"h":"m"));
	if (!iamaway)
		wprintf(win_buddy, BUDDY_TEXT, 1, "[%16.16s]", naimses.sn);
	else
		wprintf(win_buddy, BUDDY_AWAY, 1, "[%16.16s]", naimses.sn);
}

int	bonline(char *buddy) {
	buddywin_t	*bwin = curbwin;

	assert(buddy != NULL);
	assert(bwin != NULL);
	if (aimcmp(buddy, bwin->winname) == 0)
		return(1);
	while ((bwin = bwin->next) != curbwin)
		if (aimcmp(buddy, bwin->winname) == 0)
			return(1);
	return(0);
}

buddywin_t	*bgetwin(const char *buddy) {
	buddywin_t	*bwin = curbwin;

	assert(buddy != NULL);
	assert(bwin != NULL);
	if (aimcmp(buddy, bwin->winname) == 0)
		return(bwin);
	while ((bwin = bwin->next) != curbwin)
		if (aimcmp(buddy, bwin->winname) == 0)
			return(bwin);
	return(NULL);
}

void	bremove(buddywin_t *bwin) {
	assert(bwin != NULL);
	assert(bwin->win != NULL);
	assert(bwin->next != bwin);
	delwin(bwin->win);
	free(bwin);
}

#define NWIN(WIN)	newwin(faimconf.WIN.widthy, faimconf.WIN.widthx, \
	faimconf.WIN.starty, faimconf.WIN.startx);
void	bnewwin(char *buddy) {
	buddywin_t	*bwin = NULL;
//	int	sortby = secs_getvar_int("sortby");
	int	sortby = faimconf.sortby;

	assert(buddy != NULL);
	bwin = calloc(1, sizeof(buddywin_t));
	assert(bwin != NULL);
	bwin->win = NWIN(wstatus);
	assert(bwin->win != NULL);
	winitwin(bwin->win, IMWIN_BGCOLOR);
	bzero(bwin->winname, 13);
	strncpy(bwin->winname, buddy, MAXSNLEN);
	bwin->isaway = bwin->keepafterso = bwin->notbuddy
		= bwin->waiting = 0;
	bwin->offline = 1;

	if (sortby == 0) {
		bwin->next = curbwin->next;
		curbwin->next = bwin;
	} else if (sortby == 1) {
		buddywin_t	*srchbwin = &nullbwin;

		while ((srchbwin->next != &nullbwin) &&
			(aimcmp(srchbwin->next->winname, bwin->winname) != 1))
			srchbwin = srchbwin->next;
		bwin->next = srchbwin->next;
		srchbwin->next = bwin;
	} else if (sortby == 2) {
		bwin->next = nullbwin.next;
		nullbwin.next = bwin;
	}
}

void	bcoming(char *buddy, int isaway, int isidle) {
	time_t	t = 0;
	struct tm	*tmptr = NULL;
	buddywin_t	*bwin = NULL;
	buddylist_t	*blist = NULL;

	assert(buddy != NULL);
	t = time(NULL);
	assert(t != 0);
	tmptr = localtime(&t);
	assert(tmptr != NULL);

	if ((blist = rgetlist(&faimconf, buddy)) == NULL) {
		printf("Adding %s to your buddy list.\n", buddy);
		blist = raddbuddy(&faimconf, buddy, "User", NULL);
	}
	if ((bwin = bgetwin(buddy)) == NULL) {
		bnewwin(buddy);
		bwin = bgetwin(buddy);
		assert(bwin != NULL);
	}
	if (bwin->offline == 1) {
		bwin->offline = 0;
		if (*(blist->notes) != 0)
			printf("%s %s (%s) is now online =)\n",
				blist->group, buddy, blist->notes);
		else
			printf("%s %s is now online =)\n", blist->group,
				buddy);
		wprintf(bwin->win, IMWIN_STATUS, 1,
			"[%02i:%02i:%02i] %s %s is now online =)\n",
			tmptr->tm_hour, tmptr->tm_min, tmptr->tm_sec, 
			blist->group, buddy);
		if (secs_getvar_int("beeponsignon") == 1)
			beep();
	} else {
		if ((isaway == 1) && (bwin->isaway == 0)) {
			bwin->isaway = 1;
			wprintf(bwin->win, IMWIN_STATUS, 1,
				"[%02i:%02i:%02i] %s is now away.\n",
				tmptr->tm_hour, tmptr->tm_min,
				tmptr->tm_sec, buddy);
//			aim_getinfo(nfaim_sess, nfaim_conn, buddy,
//				AIM_GETINFO_AWAYMESSAGE);
		} else if ((isaway == 0) && (bwin->isaway == 1)) {
			bwin->isaway = 0;
			wprintf(bwin->win, IMWIN_STATUS, 1,
				"[%02i:%02i:%02i] %s is no longer away!\n",
				tmptr->tm_hour, tmptr->tm_min,
				tmptr->tm_sec, buddy);
		}

		if ((isidle == 1) && (bwin->isidle == 0)) {
			bwin->isidle = 1;
			wprintf(bwin->win, IMWIN_STATUS, 1,
				"[%02i:%02i:%02i] %s is now idle.\n",
				tmptr->tm_hour, tmptr->tm_min,
				tmptr->tm_sec, buddy);
		} else if ((isidle == 0) && (bwin->isidle == 1)) {
			bwin->isidle = 0;
			wprintf(bwin->win, IMWIN_STATUS, 1,
				"[%02i:%02i:%02i] %s is no longer idle!\n",
				tmptr->tm_hour, tmptr->tm_min,
				tmptr->tm_sec, buddy);
		}
	}
	bwin->notbuddy = 0;
	bupdate();
}

void	bgoing(char *buddy) {
	time_t	t = 0;
	struct tm	*tmptr = NULL;
	buddywin_t	*bwin = curbwin;
	buddylist_t	*blist = NULL;

	assert(buddy != NULL);
	assert(bwin != NULL);
	t = time(NULL);
	assert(t != 0);
	tmptr = localtime(&t);
	assert(tmptr != NULL);

	if ((blist = rgetlist(&faimconf, buddy)) != NULL) {
		if (*(blist->notes) != 0)
			printf("%s %s (%s) has just logged off :(\n",
				blist->group, buddy, blist->notes);
		else
			printf("%s %s has just logged off :(\n", 
				blist->group, buddy);
	}

	do {
		if (aimcmp(buddy, bwin->winname) == 0) {
			if (bwin->keepafterso == 1) {
				bwin->offline = 1;
				wprintf(bwin->win, IMWIN_STATUS, 1,
 					"[%02i:%02i:%02i] %s %s has just"
					" logged off :(\n",
					tmptr->tm_hour, tmptr->tm_min,
					tmptr->tm_sec, blist->group, 
					buddy);
				bupdate();
				if (secs_getvar_int("beeponsignon") == 1)
					beep();
			} else {
				buddywin_t	*bbefore = bwin->next,
						*bnext = bwin->next;

				while (bbefore->next != bwin)
					bbefore = bbefore->next;
				if (curbwin == bwin)
					curbwin = bnext;
				bremove(bwin);
				bbefore->next = bnext;
				bupdate();
				touchwin(curbwin->win);
			}
			return;
		} 
	} while ((bwin = bwin->next) != curbwin);
	printf("I just received a signoff for %s, who is not in your"
		" active buddy list.\n", buddy);
	bupdate();
}

void	bclearall(void) {
	buddywin_t	*bwin, *bnext = nullbwin.next;
	buddylist_t	*blist = NULL;

	assert(bnext != NULL);
	debug_printf("\tbclearall(), curbwin = %p\n", curbwin);
	curbwin = &nullbwin;
	debug_printf("\tbclearall(), curbwin = %p\n", curbwin);
	while ((bwin = bnext) != &nullbwin) {
		blist = rgetlist(&faimconf, bwin->winname);
		bnext = bwin->next;
//		debug_printf("blist = %p, bwin = %p, bnext = %p\n", blist,
//			bwin, bnext);
		if (bwin->ischat)
			continue;
		if (bwin->keepafterso == 1) {
			struct tm	*tmptr;
			time_t	t;

			t = time(NULL);
			assert(t != 0);
			tmptr = localtime(&t);
			assert(tmptr != NULL);
			bwin->offline = 1;
			wprintf(bwin->win, IMWIN_STATUS, 1,
				"[%02i:%02i:%02i] %s %s has just"
				" logged off :(\n",
				tmptr->tm_hour, tmptr->tm_min,
				tmptr->tm_sec, blist->group, 
				bwin->winname);
		} else {
			buddywin_t	*bbefore = bwin->next;

			while (bbefore->next != bwin)
				bbefore = bbefore->next;
			bremove(bwin);
			bbefore->next = bnext;
		}
		bupdate();
	}
	debug_printf("\tbclearall(), curbwin = %p\n", curbwin);
}
