/*  _ __   __ _ ___ __  __
** | '_ \ / _` |_ _|  \/  | naim
** | | | | (_| || || |\/| | Copyright 1998-2000 Daniel Reed <n@ml.org>
** |_| |_|\__,_|___|_|  |_| ncurses-based, FAIM-based AIM client
** 
** FAIM Copyright 1998 Adam Fritzler <afritz@iname.com>
** FAIM uses code Copyright 1998 Brock Wilcox <awwaiid@iname.com>
*/
#include <naim.h>
#include <pwd.h>

extern WINDOW	*win_status;
extern buddywin_t	nullbwin;
extern faimconf_t	faimconf;
extern int	iamaway, gotdumped;
extern naimses_t	naimses;

nFIRE_HANDLER(naim_buddylist) {
	va_list	msg;
	char	*screenname;

	va_start(msg, connection);
	screenname = va_arg(msg, char *);
	va_end(msg);

	raddbuddy(&faimconf, screenname, "Pre-existing buddy", NULL);
}

nFIRE_HANDLER(naim_budprof) {
	buddylist_t	*buddy = faimconf.buddies;
	ignorelist_t	*idiot = faimconf.idiots;

	printf("Uploading buddy list, block list, and profile (%i)...\n",connection);
	
	while (buddy != NULL) {
		firetalk_im_internal_add_buddy(sess, proto, connection,
			buddy->screenname);
		buddy = buddy->next;
	}

	while (idiot != NULL) {
		if (strcasecmp(idiot->notes, "block") == 0)
			firetalk_im_internal_add_deny(sess, proto, connection,
				idiot->screenname);
		idiot = idiot->next;
	}

	if (faimconf.profile != NULL)
		firetalk_set_info(sess, proto, connection, faimconf.profile);
	else
		firetalk_set_info(sess, proto, connection,
			"On two occasions I have been asked [by members of Parliament], 'Pray, Mr. Babbage, if you put into the machine wrong figures, will the right answers come out?' I am not able rightly to apprehend the kind of confusion of ideas that could provoke such a question. -- Charles Babbage<br>"
			"&nbsp;<br>"
			"naim-0.10.1 / FireTalk-" LIBFIRETALK_VERSION "<br>"
			"Visit <a href=\"http://dev.n.ml.org/\">http://dev.n.ml.org/</a> for the latest naim."
		);

}

nFIRE_HANDLER(naim_buddy_idle) {
	va_list	msg;
	char	*who;
	long	idletime;

	va_start(msg, connection);
	who = va_arg(msg, char *);
	idletime = va_arg(msg, long);
	va_end(msg);

//	printf("Got idle callback for %s (%li)\n", who, idletime);

	if (idletime >= 10)
		bcoming(who, -1, 1);
	else
		bcoming(who, -1, 0);
}

nFIRE_HANDLER(naim_buddy_away) {
	va_list	msg;
	char	*who;

	va_start(msg, connection);
	who = va_arg(msg, char *);
	va_end(msg);

//	printf("Got away callback for %s\n", who);

	bcoming(who, 1, -1);
}

nFIRE_HANDLER(naim_buddy_unaway) {
	va_list	msg;
	char	*who;

	va_start(msg, connection);
	who = va_arg(msg, char *);
	va_end(msg);

//	printf("Got unaway callback for %s\n", who);

	bcoming(who, 0, -1);
}

nFIRE_HANDLER(naim_buddy_coming) {
	va_list	msg;
	char	*who;

	va_start(msg, connection);
	who = va_arg(msg, char *);
	va_end(msg);

//	printf("Got coming callback for %s\n", who);

	bcoming(who, 0, 0);
}

nFIRE_HANDLER(naim_buddy_going) {
	va_list	msg;
	char	*who;

	va_start(msg, connection);
	who = va_arg(msg, char *);
	va_end(msg);

//	printf("Got going callback for %s\n", who);

	bgoing(who);
}

nFIRE_HANDLER(naim_im_handle) {
	va_list	msgg;
	buddywin_t	*bwin;
	time_t	t = 0;
	struct tm	*tmptr = NULL;
	ignorelist_t	*idiot = NULL;
	char	*screenname, *msg = NULL;
	int	isautoreply = 0;

	t = time(NULL);
	assert(t != 0);
	tmptr = localtime(&t);
	assert(tmptr != NULL);
	idiot = faimconf.idiots;

	va_start(msgg, connection);
	screenname = va_arg(msgg, char *);
	isautoreply = va_arg(msgg, char);
	msg = va_arg(msgg, char *);
	va_end(msgg);

//	if ((strcasecmp(screenname, naimses.sn) == 0)
	if ((aimcmp(screenname, naimses.sn) == 0)
		&& (strncmp(msg, "<!--PING:", 9) == 0)) {
		naimses.lag = t - atoi(msg+9);
		bupdate();
		debug_printf("naim_im_handle: resetting lag to %i\n",
			naimses.lag);
		return;
	}

	logim(screenname, naimses.sn, msg);

	while (idiot != NULL) {
		if (aimcmp(screenname, idiot->screenname) == 0) {
			if ((t - idiot->informed) > 1200) {
				char	igbuf[1200];

				snprintf(igbuf, sizeof(igbuf), "You are"
					" on my ignore list: <font"
					" size=+1 absz=14>%s"
					"</font><br>\n"
					"Your IM was not received by me,"
					" it was ignored. You will receive"
					" no more IM's from me (at least"
					" for 20 minutes).<br>\n<font"
					" size=-1 absz=10>This IM"
					" is automatic.</font>",
					idiot->notes);
				naim_send_im_away(screenname, igbuf);
				idiot->informed = t;
			}
			return;
		}
		idiot = idiot->next;
	}
	if (!iamaway && (secs_getvar_int("beeponim") == 1))
		beep();
	if (secs_getvar_int("autobuddy") == 1) {
		if (secs_getvar_int("syncbuddy") == 1)
			firetalk_im_add_buddy(sess, proto, connection, screenname);
		if (bgetwin(screenname) == NULL)
			bnewwin(screenname);
	}
	if ((bwin = bgetwin(screenname)) == NULL) {
		nullbwin.waiting = 1;
		if (isautoreply == FALSE) {
			if (strncmp(msg, "/me ", 4) == 0)
				hwprintf(win_status, IMWIN_TEXT_R,
					"[%02i:%02i:%02i] *> <B>%s</B> %s<br>",
					tmptr->tm_hour, tmptr->tm_min,
					tmptr->tm_sec, screenname, msg+4);
			else
				hwprintf(win_status, IMWIN_TEXT_R,
					"[%02i:%02i:%02i] [<B>%s</B>] %s<br>",
					tmptr->tm_hour, tmptr->tm_min,
					tmptr->tm_sec, screenname, msg);
			if (iamaway && (*secs_getvar("awaymsg") != 0)
				&& ((t - nullbwin.informed) > 600)
				&& (aimcmp(screenname,
				   naimses.sn) != 0)) {
				sendaway(screenname);
				nullbwin.informed = t;
			}
		} else
			hwprintf(win_status, IMWIN_TEXT_R,
				"[%02i:%02i:%02i] [<B>%s</B> is away] %s<br>",
				tmptr->tm_hour, tmptr->tm_min,
				tmptr->tm_sec, screenname, msg);
		bupdate();
	} else {
		bwin->keepafterso = bwin->waiting = 1;
		if (isautoreply == FALSE) {
#if 0
			if (bwin->cryptkey != NULL) {
				char	*msgsave = msg;

				msg = calloc(2*strlen(msg)+1, 1);
				naim_unxor(msg, msgsave, bwin->cryptkey);
			}
#endif
			if (strncmp(msg, "/me ", 4) == 0)
				hwprintf(bwin->win, IMWIN_TEXT_R,
					"[%02i:%02i:%02i] * <B>%s</B> %s<br>", 
					tmptr->tm_hour, tmptr->tm_min,
					tmptr->tm_sec, screenname, msg+4);
			else
				hwprintf(bwin->win, IMWIN_TEXT_R,
					"[%02i:%02i:%02i] <B>%s</B>: %s<br>", 
					tmptr->tm_hour, tmptr->tm_min,
					tmptr->tm_sec, screenname, msg);
			if (iamaway && (*secs_getvar("awaymsg") != 0)
				&& ((t - bwin->informed) > 600)
				&& (aimcmp(screenname,
				   naimses.sn) != 0)) {
				sendaway(screenname);
				bwin->informed = t;
			}
			if (bwin->cryptkey != NULL)
				free(msg);
		} else
			hwprintf(bwin->win, IMWIN_TEXT_R,
				"[%02i:%02i:%02i] <B>%s</B> is away: %s<br>", 
				tmptr->tm_hour, tmptr->tm_min,
				tmptr->tm_sec, screenname, msg);
		bupdate();
	}
}

nFIRE_HANDLER(naim_error_msg) {
	va_list	msg;
	int	error;
	char	*target;
	buddywin_t	*bwin;

	va_start(msg, connection);
	error = va_arg(msg, int);
	target = va_arg(msg, char *);
	va_end(msg);

	if ((target != NULL) && ((bwin = bgetwin(target)) != NULL)) {
		time_t	t = time(NULL);
		struct tm	*tmptr;

		tmptr = localtime(&t);
		wprintf(bwin->win, IMWIN_STATUS, 1,
			"[%02i:%02i:%02i] ERROR: %s\n", tmptr->tm_hour,
			tmptr->tm_min, tmptr->tm_sec, firetalkerrors[error]);
		bwin->waiting = 1;
		bupdate();
	} else
		printf("ERROR: %s: %s\n", target, firetalkerrors[error]);
};

nFIRE_HANDLER(naim_error_disconnect) {
	va_list	msg;
	int	error;

	va_start(msg, connection);
	error = va_arg(msg, int);
	va_end(msg);

	printf("ERROR: Disconnect: %s\n", firetalkerrors[error]);
	gotdumped = 1;
}

nFIRE_HANDLER(naim_userinfo_handler) {
	va_list	msg;
	buddywin_t	*bwin;
	char	*SN, *info;
	int	warning, idle;

	va_start(msg, connection);
	SN = va_arg(msg, char *);
	info = va_arg(msg, char *);
	warning = va_arg(msg, int);
	idle = va_arg(msg, int);
	va_end(msg);

	if ((SN != NULL) && ((bwin = bgetwin(SN)) != NULL)) {
		hwprintf(bwin->win, STAT_TEXT,
			"          <B>Class</B>: [unknown]<br>");
		if (warning/10.0 > 0)
		  hwprintf(bwin->win, STAT_TEXT,
			"  <B>Warning level</B>: %.1f<br>",
			warning/10.0);
		if (idle > 0)
		  hwprintf(bwin->win, STAT_TEXT,
			"      <B>Idle time</B>: <B>%i</B> minutes<br>",
			idle);
		if (info != NULL) {
		  hwprintf(bwin->win, STAT_TEXT,
			"        <B>Profile</B>:<br>");
		  hwprintf(bwin->win, STAT_TEXT, "%s<br><hr>", info);
		}
		bwin->waiting = 1;
		bupdate();
	} else {
		printf("Information about %s:\n", SN);
		printf("          Class: [unknown]\n");
		if (warning/10 > 0)
			printf("  Warning level: %.1f%%\n", warning/10.0);
		if (idle > 0)
			printf("      Idle time: %i minutes\n", idle);
		if (info != NULL) {
			printf("Profile:\n");
			hwprintf(win_status, STAT_TEXT, "%s<br><hr>", info);
		}
	}
}
