/*  _ __   __ _ ___ __  __
** | '_ \ / _` |_ _|  \/  | naim
** | | | | (_| || || |\/| | Copyright 1998, 1999 Daniel Reed <n@ml.org>
** |_| |_|\__,_|___|_|  |_| ncurses-based, FAIM-based AIM client
** 
** FAIM Copyright 1998 Adam Fritzler <afritz@iname.com>
** FAIM uses code Copyright 1998 Brock Wilcox <awwaiid@iname.com>
*/
#include <naim.h>
#include <sys/stat.h>

extern WINDOW	*win_status;
extern buddywin_t	nullbwin;
extern faimconf_t	faimconf;
time_t	idlesince = 0;
int	iamaway = 0;
extern naimses_t	naimses;
extern void	*nfire_c;
extern int	nfire_p;

void	logim(const char *source, const char *target, const char *msg) {
	static FILE	*logfile;
	time_t	t = 0;

	if (faimconf.nolog == 1)
		return;
	t = time(NULL);
	assert(t != 0);
	if ((source == NULL) && (target == NULL) && (msg != NULL)
		&& (*msg != 0)) {
		struct tm	*tmptr = NULL;

		tmptr = localtime(&t);
		assert(tmptr != NULL);
		if ((logfile = fopen(msg, "a")) == NULL)
			printf("Unable to open \"%s\": %m", msg);
		fchmod(fileno(logfile), 0600);
		fprintf(logfile, "\n*** Log opened %02i-%02i-%04i at"
			" %02i:%02i.\n", tmptr->tm_mon+1, tmptr->tm_mday,
			1900+tmptr->tm_year, tmptr->tm_hour,
			tmptr->tm_min);
		return;
	}
	assert(source != NULL);
	assert(*source != 0);
	assert(target != NULL);
	assert(*target != 0);
	assert(target != NULL);
	assert(*target != 0);
	fprintf(logfile, "%lu %10s | %10s | %s\n", t, source, target,
		msg);
	fflush(logfile);
}

#if 0
char	*naim_unxor(char *msg, const char *inmsg, const char *key) {
}

char	*naim_xor(char *msg, const char *inmsg, const char *key) {
	int	mlen = strlen(inmsg),
		klen = strlen(key), i, j = 0;

	while (j < mlen)
		for (i = 0; (i < klen) && (j < mlen); i++, j += 2) {
//			msg[j] = inmsg[j] ^ key[i];
			char	t;

			if ((t = (inmsg[j/2] ^ key[i]) >> 4) < 10)
				msg[j] = t + '0';
			else
				msg[j] = t + 'A';
			if ((t = ((inmsg[j/2] ^ key[i])&0x0F)) < 10)
				msg[j+1] = t + '0';
			else
				msg[j+1] = t + 'A';
		}
	return(msg);
}
#endif

void	naim_send_im(/*const*/ char *SN, /*const*/ char *msg, const char *key) {
	updateidletime();
#if 0
	if ((key != NULL) && (*key != 0)) {
		char	*cmsg = calloc(2*strlen(msg)+1, 1);

		naim_xor(cmsg, msg, key);
		aim_send_im(nfaim_sess, nfaim_conn, SN, 0, cmsg);
		free(cmsg);
	} else {
#endif
		firetalk_im_send_message(nfire_c, nfire_p, 0, SN, msg, 0);
//	}
}

void	naim_send_im_away(/*const*/ char *SN, /*const*/ char *msg) {
	time_t	t;
	struct tm	*tmptr;
	buddywin_t	*bwin;

	t = time(NULL);
	assert(t != 0);
	tmptr = localtime(&t);
	assert(tmptr != NULL);
	if ((bwin = bgetwin(SN)) == NULL)
		printf("Sent away IM to %s (%s)\n", SN, msg);
	else
		hwprintf(bwin->win, IMWIN_TEXT_U,
			"[%02i:%02i:%02i] <B>%s</B> is away: %s<br>", 
			tmptr->tm_hour, tmptr->tm_min, tmptr->tm_sec,
			naimses.sn, msg);
	firetalk_im_send_message(nfire_c, nfire_p, 0, SN, msg, 1);
}

void	setaway(void) {
	iamaway = 1;
	idlesince = time(NULL);
	printf("You are now away--hurry back!\n");
	firetalk_set_away(nfire_c, nfire_p, 0, secs_getvar("awaymsg"));
}

void	unsetaway(void) {
	iamaway = 0;
	idlesince = 0;
	printf("You are no longer away--welcome back =)\n");
	firetalk_set_away(nfire_c, nfire_p, 0, NULL);
	updateidletime();
}

void	sendaway(char *SN) {
	char	buf[1124];
	struct tm	*tmptr = NULL;

	if (iamaway == 0)
		return;
	assert(idlesince != 0);
	tmptr = localtime(&idlesince);
	assert(tmptr != NULL);
	snprintf(buf, sizeof(buf), "[Away since %02i/%02i at %02i:%02i] %s",
		1+tmptr->tm_mon, tmptr->tm_mday, tmptr->tm_hour,
		tmptr->tm_min, secs_getvar("awaymsg"));
	naim_send_im_away(SN, buf);
}
