/*  _ __   __ _ ___ __  __
** | '_ \ / _` |_ _|  \/  | naim
** | | | | (_| || || |\/| | Copyright 1998, 1999 Daniel Reed <n@ml.org>
** |_| |_|\__,_|___|_|  |_| ncurses-based, FAIM-based AIM client
** 
** FAIM Copyright 1998 Adam Fritzler <afritz@iname.com>
** FAIM uses code Copyright 1998 Brock Wilcox <awwaiid@iname.com>
*/
#include <naim.h>

extern WINDOW	*win_input, *win_buddy;
extern buddywin_t	*curbwin, nullbwin;
extern faimconf_t	faimconf;
extern int	wsetup_called;
int	stayconnected, showrcedit = 0, gotdumped;
naimses_t      naimses;
void	*nfire_c;
int	nfire_p;

int	main(int argc, char **args) {
	time_t	lastcycle = 0;
	char	faimrcpath[256];

//	signal(SIGWINCH, SIG_IGN);
	if (getenv("FAIMRC") != NULL) {
		strncpy(faimrcpath, getenv("FAIMRC"),
			sizeof(faimrcpath)-1);
		faimrcpath[sizeof(faimrcpath)-1] = 0;
	} else
		snprintf(faimrcpath, sizeof(faimrcpath), "%s/.faimrc",
			getenv("HOME"));

	secs_init();
	initscr();

	wsetup_called = 2;

	printf("Reading %s\n", faimrcpath);
	if (naim_read_config(faimrcpath) == 0) {
		wshutitdown();
		printf("Error %i reading %s: %s\n", errno,
			faimrcpath, strerror(errno));
		exit(-1);
	} else
		endwin();
	wsetup_called = 0;
	wsetup();
	bsetup();
	gotkey(0);	// initialize gotkey() buffer
	updateidletime();
	wcols(&faimconf);
	if (faimconf.nolog == 0) {
		if (*(faimconf.logfile) == 0)
			strncpy(faimconf.logfile, "naim.log",
				sizeof(faimconf.logfile)-1);
		printf("Opening log file (%s)\n", faimconf.logfile);
		logim(NULL, NULL, faimconf.logfile);
	} else if (*(faimconf.logfile) != 0)
		printf("You have logging disabled, but you specified a log file.\n");
	printf("Contact Daniel Reed <n@ml.org> if you'd like\n");
	printf("information about naim.\n");
	wrefresh(nullbwin.win);

	naimses.online = naimses.connected = 0;
	nfire_c = firetalk_create_handle();
	nfire_p = FP_AIMTOC;

	firetalk_register_callback(nfire_c, FC_DISCONNECT,
		naim_error_disconnect);
	firetalk_register_callback(nfire_c, FC_ERROR,
		naim_error_msg);
	firetalk_register_callback(nfire_c, FC_IM_GETMESSAGE,
		naim_im_handle);
	firetalk_register_callback(nfire_c, FC_IM_BUDDYONLINE,
		naim_buddy_coming);
	firetalk_register_callback(nfire_c, FC_IM_BUDDYOFFLINE,
		naim_buddy_going);
	firetalk_register_callback(nfire_c, FC_IDLEINFO,
		naim_buddy_idle);
	firetalk_register_callback(nfire_c, FC_IM_BUDDYAWAY,
		naim_buddy_away);
	firetalk_register_callback(nfire_c, FC_IM_BUDDYUNAWAY,
		naim_buddy_unaway);
	firetalk_register_callback(nfire_c, FC_GOTINFO,
		naim_userinfo_handler);
	firetalk_register_callback(nfire_c, FC_IM_LISTBUDDY,
		naim_buddylist);
	firetalk_register_callback(nfire_c, FC_DOINIT,
		naim_budprof);

	do {
		int	keepgoing;
		fd_set	rfd;

		printf("Authenticating %s.\n", faimconf.screenname);
		wrefresh(nullbwin.win);
		if (firetalk_signon(nfire_c, nfire_p, 0, NULL, 0, faimconf.screenname,
			faimconf.password)) {
			firetalk_perror("firetalk_signon");
			wrefresh(nullbwin.win);
			wshutitdown();
			exit(1);
		}

/*		firetalk_list_buddies(nfire_c, nfire_p); */

		printf("Entering event loop.\n");
		wrefresh(nullbwin.win);

		naimses.lag = naimses.sn[MAXSNLEN] = 0;
		strncpy(naimses.sn, faimconf.screenname, MAXSNLEN);

		debug_printf("Entering event loop.\n");
		keepgoing = stayconnected = 1;
		gotdumped = 0;

		naimses.online = time(NULL);
		naimses.connected = 1;

		while (stayconnected && keepgoing && !gotdumped) {
			struct timeval	timeout;
			time_t	now;

			timeout.tv_sec = 50;
			timeout.tv_usec = 0;

			FD_ZERO(&rfd);
			FD_SET(STDIN_FILENO, &rfd);
			if (firetalk_select_custom(nfire_c, 1,
				&rfd, NULL, NULL, &timeout) < 0) {
				printf("Red alert!\n");
				wshutitdown();
				exit(0);
			}

			now = time(NULL);
			now -= (now%60);
			if ((now - lastcycle) >= 60)
				event_handle(lastcycle = now);

			if (FD_ISSET(STDIN_FILENO, &rfd))
				gotkey(getch());

			wnoutrefresh(win_buddy);
			wnoutrefresh(curbwin->win);
			wnoutrefresh(win_input);
			doupdate();
		}
		naimses.connected = 0;
		if (stayconnected)
			printf("Connection to AIM lost, attempting to reconnect.\n");
		bclearall();
		wnoutrefresh(win_buddy);
		wnoutrefresh(nullbwin.win);
		wnoutrefresh(win_input);
		doupdate();
		printf("Closing all lingering AIM connection.\n");
//		aim_logoff(nfaim_sess);
	} while (stayconnected);

//	aim_logoff(nfaim_sess);
	wshutitdown();
	exit(0);
}
